/*
 * J2ME Compass
 * Copyright (C) 2006 Dana Peters
 * http://www.qcontinuum.org/compass
 * 
 * Changed 2009 by Vedran Vidovic.
 * Removed henson.midp.Float class for performance reasons.
 * Removed code unused by SgTimer.
 */

package org.qcontinuum.compass;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.util.Date;

import javax.microedition.rms.RecordStore;
import javax.microedition.rms.RecordStoreException;

import org.qcontinuum.astro.EarthPosition;


public class Location {
    
    public String Name = "";
    public short LatMinutes;
    public short LongMinutes;
    public TimeZone timeZone = new TimeZone();
    public boolean dstEnabled = true;

    public Location() {
    }

    public Location(Location location) {
        Name = location.Name;
        LatMinutes = location.LatMinutes;
        LongMinutes = location.LongMinutes;
        if (location.timeZone == null)
            timeZone = null;
        else
            timeZone = new TimeZone(location.timeZone);
        dstEnabled = location.dstEnabled;
    }
    
    public Location(String name) {
        Name = name;
        load();
    }

    public Location(DataInputStream inputStream) throws IOException {
        load(inputStream);
    }
    
    public void updateLocation(final Location location) {
      this.Name = location.Name;
      this.LatMinutes = location.LatMinutes;
      this.LongMinutes = location.LongMinutes;
      this.timeZone = location.timeZone;
      this.dstEnabled = location.dstEnabled;
    }
        
    public EarthPosition getEarthPosition() {
        return new EarthPosition(LatMinutes, LongMinutes);
    }

    public Date getGmt(Date localDate) {
        return timeZone.getGmt(localDate, dstEnabled);
    }
    
    public Date getLocalTime(Date gmtDate) {
        return timeZone.getLocalTime(gmtDate, dstEnabled);
    }
    
    public int getStdOffset() {
        return timeZone.getStdOffset();
    }
    
    public int getDstOffset(Date localDate) {
        return timeZone.getDstOffset(localDate, dstEnabled);
    }

    public void load(String name) {
        Name = name;
        load();
    }
 
    private void load() {
        RecordStore recordStore;
        try {
	    recordStore = RecordStore.openRecordStore(Name, false);
            byte[] bytes = recordStore.getRecord(1);
            ByteArrayInputStream bais = new ByteArrayInputStream(bytes);
            DataInputStream inputStream = new DataInputStream(bais);
            load(inputStream);
            inputStream.close();
            recordStore.closeRecordStore();
        }
        catch (Exception exception) {
            Name = null;
            LatMinutes = LongMinutes = 0;
            timeZone = new TimeZone();
            return;
        }
    }
    
    public void load(DataInputStream inputStream) throws IOException {
        Name = restoreNull(inputStream.readUTF());
        LatMinutes = inputStream.readShort();
        LongMinutes = inputStream.readShort();
        if (inputStream.readBoolean())
            timeZone = new TimeZone(inputStream);
        else
            timeZone = null;
        dstEnabled = inputStream.readBoolean();
    }

    public void save() {
        RecordStore recordStore;
        try {
            recordStore = RecordStore.openRecordStore(Name, true);
        }
        catch (RecordStoreException rsc) {
            return;
        }
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        DataOutputStream outputStream = new DataOutputStream(baos);
        try {
            save(outputStream);
            byte[] bytes = baos.toByteArray();
            if (recordStore.getNumRecords() == 0)
                recordStore.addRecord(bytes, 0, bytes.length);
            else
                recordStore.setRecord(1, bytes, 0, bytes.length);
            outputStream.close();
            recordStore.closeRecordStore();
        }
        catch (Exception exception) {
        }
    }

    public void save(DataOutputStream outputStream) throws IOException {
        outputStream.writeUTF(removeNull(Name));
        outputStream.writeShort(LatMinutes);
        outputStream.writeShort(LongMinutes);
        outputStream.writeBoolean(timeZone != null);
        if (timeZone != null)
            timeZone.save(outputStream);
        outputStream.writeBoolean(dstEnabled);
    }
    
    private String removeNull(String text) {
        return text != null ? text : "";
    }
    
    private String restoreNull(String text) {
        return text.length() > 0 ? text : null;
    }
    
    public short getLatDegs() {
      return Location.getDegPart(LatMinutes);
    }

    public short getLatMins() {
      return Location.getMinPart(LatMinutes);
    }
    
    public short getLongDegs() {
      return Location.getDegPart(LongMinutes);
    }
    
    public short getLongMins() {
      return Location.getMinPart(LongMinutes);
    }
    
    public static short getDegPart(short minutes) {
      return (short)(minutes / 60);
    }

    public static short getMinPart(short minutes) {
      return (short)(minutes % 60);
    }
    
    public String toString() {
      return Name + " " + getEarthPosition().toString();
    }
}
